package com.boot.bus.sqlsync.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.boot.bus.sqlsync.enums.DataSyncOperationEnum;
import com.boot.bus.sqlsync.enums.DataSyncTypeEnum;
import com.boot.bus.sqlsync.message.DataSyncMessage;
import com.boot.bus.sqlsync.service.infc.*;
import com.yeejoin.amos.fas.dao.entity.ContingencyPlanInstance;
import com.yeejoin.amos.fas.datasync.bo.ContingencyOriginalDataSyncBo;
import com.yeejoin.amos.fas.datasync.bo.PlanDetailSyncBo;
import com.yeejoin.amos.fas.datasync.bo.PlanOperationRecordSyncBo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author keyong
 * @title: MqttReceiveServiceImpl
 *
 * <pre>
 * &#64;description: 增量数据处理
 *         </pre>
 * @date 2020/11/3 13:39
 */
@Slf4j
@Service
public class MqttReceiveServiceImpl implements MqttReceiveService {

    @Autowired
    private IContingencyPlanDetailService contingencyPlanDetailService;

    @Autowired
    private IPlanOperationRecordService planOperationRecordService;

    @Autowired
    private IContingencyOriginalDataService contingencyOriginalDataService;

    @Autowired
    private IContingencyPlanInstance contingencyPlanInstance;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void handlerMqttIncrementMessage(String topic, String message) {
        log.info(String.format("收到mqtt消息：%s", message));
        DataSyncMessage dataSyncMessage = JSON.parseObject(message, DataSyncMessage.class);
        List<Serializable> data = dataSyncMessage.getData();
        DataSyncTypeEnum type = dataSyncMessage.getType();
        DataSyncOperationEnum operation = dataSyncMessage.getOperation();
        assert dataSyncMessage.getType() != null && dataSyncMessage.getOperation() != null : "同步消息体信息不足";
        if (data == null || data.isEmpty()) {
            return;
        }
        switch (type) {
            case CONTINGENCY_PLAN_DETAIL: {
                switch (operation) {
                    case DELETE: {
                        contingencyPlanDetailService.astDeleteByIds(data.stream().map(x -> ((JSONObject) x).toJavaObject(PlanDetailSyncBo.class).getId()).collect(Collectors.toList()));
                        break;
                    }
                    default: {
                        contingencyPlanDetailService.astSaveOrUpdateBatch(data.stream().map(x -> ((JSONObject) x).toJavaObject(PlanDetailSyncBo.class)).collect(Collectors.toList()));
                    }
                }
                break;
            }
            case CONTINGENCY_PLAN_OPERATION_RECORD: {
                switch (operation) {
                    case DELETE: {
                        planOperationRecordService.astDeleteByIds(data.stream().map(x -> ((JSONObject) x).toJavaObject(PlanOperationRecordSyncBo.class).getId()).collect(Collectors.toList()));
                        break;
                    }
                    default: {
                        planOperationRecordService.astSaveOrUpdateBatch(data.stream().map(x -> ((JSONObject) x).toJavaObject(PlanOperationRecordSyncBo.class)).collect(Collectors.toList()));
                    }
                }
                break;
            }
            case CONTINGENCY_ORIGINAL_DATA: {
                switch (operation) {
                    case DELETE: {
                        contingencyOriginalDataService.astDeleteByIds(data.stream().map(x -> ((JSONObject) x).toJavaObject(ContingencyOriginalDataSyncBo.class).getId()).collect(Collectors.toList()));
                        break;
                    }
                    default: {
                        contingencyOriginalDataService.astSaveOrUpdateBatch(data.stream().map(x -> ((JSONObject) x).toJavaObject(ContingencyOriginalDataSyncBo.class)).collect(Collectors.toList()));
                    }
                }
                break;
            }
            case CONTINGENCY_PLAN_INSTANCE: {
                switch (operation) {
                    case DELETE: {
                        contingencyPlanInstance.astDeleteByIds(data.stream().map(x -> ((JSONObject) x).toJavaObject(ContingencyPlanInstance.class).getId()).collect(Collectors.toList()));
                        break;
                    }
                    default: {
                        contingencyPlanInstance.astSaveOrUpdateBatch(data.stream().map(x -> ((JSONObject) x).toJavaObject(ContingencyPlanInstance.class)).collect(Collectors.toList()));
                    }
                }
                break;
            }
            case PATROL_CHECK: {
                switch (operation) {
                    case DELETE: {

                        break;
                    }
                    default: {

                    }
                }
            }
            case PATROL_CHECK_INPUT: {
                switch (operation) {
                    case DELETE: {

                        break;
                    }
                    default: {

                    }
                }
            }
            case PATROL_CHECK_SHOT: {
                switch (operation) {
                    case DELETE: {

                        break;
                    }
                    default: {

                    }
                }
            }
            case PATROL_LATENT_DANGER: {
                switch (operation) {
                    case DELETE: {

                        break;
                    }
                    default: {

                    }
                }
            }
            case PATROL_LATENT_DANGER_FLOW_RECORD: {
                switch (operation) {
                    case DELETE: {

                        break;
                    }
                    default: {

                    }
                }
            }
            case PATROL_LATENT_DANGER_PATROL: {
                switch (operation) {
                    case DELETE: {

                        break;
                    }
                    default: {

                    }
                }
            }
            case PATROL_LATENT_DANGER_PHOTO: {
                switch (operation) {
                    case DELETE: {

                        break;
                    }
                    default: {

                    }
                }
            }
            case PATROL_PLAN_TASK: {
                switch (operation) {
                    case DELETE: {

                        break;
                    }
                    default: {

                    }
                }
            }
            case PATROL_PLAN_TASK_DETAIL: {
                switch (operation) {
                    case DELETE: {

                        break;
                    }
                    default: {

                    }
                }
            }
            default:
//                Async.shutDown();
        }
    }

}
